function alarmResult=challengeAsystoleFusion(recordName,alarm_type)
%
% Last Modified: 10 Aug 2015 - Nadi Sadr
%
% alarmResult=challenge(recordName,alarm_type)
%
% Sample entry for the 2015 PhysioNet/CinC Challenge.
%
% Inputs:
%   recordName
%       String specifying the record name to process
%   alarmType
%       String specifying the alarm type. Alarm types are:
%             Asystole, Bradycardia, Tachycardia, Ventricular_Tachycardia,
%             Ventricular_Flutter_Fib
%
%
% Outputs:
%   alarmResult
%       Integer value where 0 = false alarm and 1 is a true alarm.
%
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% Dependencies:
%
%       1) This function does not requires that you have the WFDB App
%       Toolbox installed. A matlab function 'rdmat' can read the data
%       instead of using WFDB Toolbox.
%
%       2) The CHALLENGE function requires that you have downloaded the
%       challenge data 'set-p' in a subdirectory of the current directory.
%       The subdirectory should be called '/challenge/2015/set-p/' . The
%       'set-p' dataset can be downloaded from PhysioNet at:
%           http://physionet.org/physiobank/database/challenge/2015/
%
%         This dataset is used by the generateValidationSet.m script to
%         create the annotations on your traing set that will be used to
%         verify that your entry works properly in the PhysioNet testing
%         environment.
%
% Version 0.5
%
%
% Written by Qiao Li, November 10, 2014. Last Modified: Ikaro Silva
% February 11, 2015
%
%
%
% %Example using training data-
% alarmResult=challenge('./challenge/set-p/100','Asystole')
%

% Name of file containing answers
answers = 'answers.txt';

%Get all ECG, blood pressure and photoplethysmogram signals
[~,signal,Fs,siginfo]=rdmat(recordName);
alarmResult=1;

% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialise
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

description=squeeze(struct2cell(siginfo));
description=description(4,:);

% Resample signal to 125Hz
Fs=Fs(1);
if Fs~=125
    signal=resample(signal,125,Fs);
    Fs=125;
end

% Adding ECG signals
% ecg2_signal1 = signal(:,1);
% ecg2_signal = medfilt1(ecg2_signal1);
% Rpeaks_ecg2 = QRSdet_Hilbert(ecg2_signal',Fs);

% ecg5_signal1 = signal(:,2);
% ecg5_signal = medfilt1(ecg5_signal1);
% Rpeaks_ecg5 = QRSdet_Hilbert(ecg5_signal',Fs);


%***********************  Preprocessing  ********************************
ECGIIqrs = medfilt1(signal(:,1),20);              % 2 median filters to remove Baseline  with 200ms & 600 ms width
ECGII_MedFilt = medfilt1(ECGIIqrs,60);
% ECGIIqrs=sortfilt1(ECGIIraw,20,50);
% ECGIIt=sortfilt1(ECGIIqrs,60,50);
ecg2_Preproc = signal(:,1) - ECGII_MedFilt;
ecg2_signal = ecg2_Preproc;

ECGVqrs = medfilt1(signal(:,2),20);
ECGV_MedFilt = medfilt1(ECGVqrs,60);
ecg5_Preproc = signal(:,2) - ECGV_MedFilt;
ecg5_signal = ecg5_Preproc;
% *********************** NaN Modification *******************************
% Check for NaN values in the ECGs and replace them with zero(previous part of
% ECG (mirroring)), then apply Hilbert QRS detection (as the Hilbert
% algorithm stops when gets to NaN) 23/7/15 Nadi:
% [iNaN, RpeaksMod_ecg2, RpeaksNaN2, ECG2_NaN, ecg2_Preproc] = ecg_NaNModify(ecg2_Preproc, Fs);
% [iNaN5, RpeaksMod_ecg5, RpeaksNaN5, ECG5_NaN, ecg5_Preproc] = ecg_NaNModify(ecg5_Preproc, Fs);
% [RpeaksMod_ecg2] = ecg_NaNModify(ecg2_Preproc, Fs);
% [RpeaksMod_ecg5] = ecg_NaNModify(ecg5_Preproc, Fs);
iNaN = find(isnan(ecg2_Preproc));
if isempty(iNaN)
    Rpeaks_NaNecg2=QRSdet_Hilbert(ecg2_Preproc',Fs);
else
    ecg2_Preproc(iNaN)=0;
    Rpeaks_NaNecg2=QRSdet_Hilbert(ecg2_Preproc',Fs);
end
ECGIINaN=ecg2_Preproc;
% Removing the replaced zeroes from ECG signal and R peaks after QRS
% detected to the end of the signal
ecg2_Preproc(iNaN)=NaN;
% In record 123; 4 detected Rpeaks are over ECG length!: So any Rpeaks over ECG length removed
RpeaksOverECG=find(Rpeaks_NaNecg2>length(ecg2_Preproc));
Rpeaks_NaNecg2(RpeaksOverECG)=[];
RpeaksNaN=Rpeaks_NaNecg2;
% Remove the Rpeaks falling in the NaN ECG segments
kk=find(isnan(ecg2_Preproc(Rpeaks_NaNecg2)));
Rpeaks_NaNecg2(kk)=[];
Rpeaks_ecg2 = Rpeaks_NaNecg2;

% Same NaN detections and replacing zeroes for ECGV for proper performance of Hilbert QRS detection
iNaN5 = find(isnan(ecg5_Preproc));
if isempty(iNaN5)
    Rpeaks_NaNecg5=QRSdet_Hilbert(ecg5_Preproc',Fs);
else
    ecg5_Preproc(iNaN5)=0;
    Rpeaks_NaNecg5=QRSdet_Hilbert(ecg5_Preproc',Fs);
end
ECGVNaN=ecg5_Preproc;
ecg5_Preproc(iNaN5)=NaN;
RpeaksOverECGV=find(Rpeaks_NaNecg5>length(ecg5_Preproc));
Rpeaks_NaNecg5(RpeaksOverECGV)=[];
RpeaksNaN5=Rpeaks_NaNecg5;
kk5=find(isnan(ecg5_Preproc(Rpeaks_NaNecg5)));
Rpeaks_NaNecg5(kk5)=[];
Rpeaks_ecg5 = Rpeaks_NaNecg5;

%%Users can access the raw samples of the record by running the following
%command if WFDB Toolbox installed: [tm,signal]=rdsamp(recordName);
%%For more information please see the help in RDSAMP

%Run WABP on the record, which by default will analyze the first ABP, ART,
%or BP signal
N=[];
N0=[];
abp_ind=get_index(description,'ABP');
ann_abp=[];
features=[];
BEATQ=[];
R=[];

if(~isempty(abp_ind))
    ann_abp=wabp(signal(:,abp_ind),0,1);
    % Analyze the signal quality index of ABP using jSQI
    if length(ann_abp)>=3 % at least 3 abp beats detected
        [features] = abpfeature(signal(:,abp_ind),ann_abp);
        [BEATQ R] = jSQI(features, ann_abp, signal(:,abp_ind));
    end
end

%Run WABP on the record of 'PLETH' to analyze photoplethysmogram signal
ppg_ind=get_index(description,'PLETH');
ann_ppg=[];
if (~isempty(ppg_ind))
    y=quantile(signal(:,ppg_ind),[0.05,0.5,0.95]);
    ann_ppg=wabp(signal(:,ppg_ind),0,(y(3)-y(1))/120);
    % Analyze the signal quality index of PPG
    if ~isempty(ann_ppg)
        [psqi]=ppgSQI(signal(:,ppg_ind),ann_ppg);
    end
end


%Make decisions

% set valid data segment for decision making, 16s before the alarm
N_d=Fs*5*60; % alarm position
N0_d=N_d-Fs*16+1; % 16s before the alarm
ecg2_sig_sect = ecg2_signal(N0_d:N_d);
ecg5_sig_sect = ecg5_signal(N0_d:N_d);


% select the beats in the segment
n_abp_beats=intersect(find(ann_abp>=N0_d),find(ann_abp<=N_d));
n_ppg_beats=intersect(find(ann_ppg>=N0_d),find(ann_ppg<=N_d));
%Added ECG II
n_ECG2_beats=intersect(find(Rpeaks_ecg2>=N0_d),find(Rpeaks_ecg2<=N_d));
%Added ECG V
n_ECG5_beats=intersect(find(Rpeaks_ecg5>=N0_d),find(Rpeaks_ecg5<=N_d));

% hr_max_abp=NaN;
% max_rr_abp=NaN;
% max_rr_abp=NaN;
% hr_max_ppg=NaN;
% max_rr_ppg=NaN;
% hr_max_ECG2=NaN;
% max_rr_ECG2=NaN;
% % hr_max_ECG5=NaN; max_rr_ECG5=NaN;
%
%
% % calculate the heart rate
% if length(n_abp_beats)>=2
%     hr_max_abp=60*Fs/min(diff(ann_abp(n_abp_beats)));
%     max_rr_abp=max(diff(ann_abp(n_abp_beats)))/Fs;
% end
% if length(n_ppg_beats)>=2
%     hr_max_ppg=60*Fs/min(diff(ann_ppg(n_ppg_beats)));
%     max_rr_ppg=max(diff(ann_ppg(n_ppg_beats)))/Fs;
% end
%Added ECG II
if length(n_ECG2_beats)>=2
    hr_max_ECG2=60*Fs/min(diff(Rpeaks_ecg2(n_ECG2_beats)));
    max_rr_ECG2=max(diff(Rpeaks_ecg2(n_ECG2_beats)))/Fs;
else
     max_rr_ECG2=0;
end
%Added ECG V
if length(n_ECG5_beats)>=2
    hr_max_ECG5=60*Fs/min(diff(Rpeaks_ecg5(n_ECG5_beats)));
    max_rr_ECG5=max(diff(Rpeaks_ecg5(n_ECG5_beats)))/Fs;
else
   max_rr_ECG5=0; 
end


% % calculate low heart rate of 5 consecutive beats for Bradycardia
% low_hr_abp=NaN;
% low_hr_ppg=NaN;
% if length(n_abp_beats>=5)
%     for i=1:length(n_abp_beats)-4
%         low_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+4))-ann_abp(n_abp_beats(i)))/4);
%     end
% end
% low_hr_abp=min(low_hr_abp);
% if length(n_ppg_beats>=5)
%     for i=1:length(n_ppg_beats)-4
%         low_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+4))-ann_ppg(n_ppg_beats(i)))/4);
%     end
% end
% low_hr_ppg=min(low_hr_ppg);
%
% % calculate high heart reate of 17 consecutive beats for Tachycardia
% high_hr_abp=NaN;
% high_hr_ppg=NaN;
% if length(n_abp_beats>=17)
%     for i=1:length(n_abp_beats)-16
%         high_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+16))-ann_abp(n_abp_beats(i)))/16);
%     end
% end
% high_hr_abp=max(high_hr_abp);
% if length(n_ppg_beats>=17)
%     for i=1:length(n_ppg_beats)-16
%         high_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+16))-ann_ppg(n_ppg_beats(i)))/16);
%     end
% end
% high_hr_ppg=max(high_hr_ppg);


% calculate the signal quality index

if ~isempty(ann_abp)
    abpsqi=1-sum(sum(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:)))/numel(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:));
else
    abpsqi=0;
end
if ~isempty(ann_ppg)
    ppgsqi=mean(psqi(intersect(n_ppg_beats,1:length(psqi))));
else
    ppgsqi=0;
end

% Estimate the missing or NaN beats in the signals by comparing to R-R mean
rr_mean(1)    = nanmean(diff(Rpeaks_ecg2)); %rr_ecg2_mean
rr_mean(2)    = nanmean(diff(Rpeaks_ecg5)); %rr_ecg5_mean
rr_mean(3)    = nanmean(diff(ann_abp));     %rr_abp_mean
rr_mean(4)    = nanmean(diff(ann_ppg));     %rr_ppg_mean
rr_mean       = nanmean(rr_mean(:));

mx_rr_ecg2=NaN;
mx_rr_ecg5=NaN;
ecg2_rr = [];
ecg2_bpm = [];
mx_rr_ecg2 = 0;
ecg2_beats = Rpeaks_ecg2(n_ECG2_beats);
if ~isempty(ecg2_beats)
    ecg2_beats(end+1) = N_d;
    ecg2_beats        = sort(unique(ecg2_beats));
    ecg2_rr           = diff(ecg2_beats);
    ecg2_rr(end+1)    = ecg2_rr(end);
    % Remove NaN and missing Rpeaks by comparing R-R to mean RR of signals
    ecg2_rr_Outlier   = find(ecg2_rr>2*rr_mean);
    %     ecg2_rr(ecg2_rr_Outlier)=[];
    ecg2_bpm          = (60*Fs)./ecg2_rr;
    mx_rr_ecg2        = nanmax(ecg2_rr)/Fs;
else
    ecg2_rr_Outlier = [];
end

ecg5_beats = Rpeaks_ecg5(n_ECG5_beats);
ecg5_rr = [];
ecg5_bpm = [];
mx_rr_ecg5 = 0;
if ~isempty(ecg5_beats)
    ecg5_beats(end+1) = N_d;
    ecg5_beats        = sort(unique(ecg5_beats));
    ecg5_rr           = diff(ecg5_beats);
    ecg5_rr(end+1)    = ecg5_rr(end);
    % Remove NaN and missing Rpeaks by comparing R-R to mean RR of signals
    ecg5_rr_Outlier   = find(ecg5_rr>2*rr_mean);
    %     ecg5_rr(ecg5_rr_Outlier)=[];
    ecg5_bpm          = (60*Fs)./ecg5_rr;
    mx_rr_ecg5        = nanmax(ecg5_rr)/Fs;
else
    ecg5_rr_Outlier = [];
end

abp_beats = ann_abp(n_abp_beats);
abp_rr = [];
abp_bpm = [];
mx_rr_abp = 0;
if ~isempty(abp_beats)
    abp_beats(end+1) = N_d;
    abp_beats        = sort(unique(abp_beats));
    abp_rr           = diff(abp_beats);
    abp_rr(end+1)    = abp_rr(end);
    abp_bpm          = (60*Fs)./abp_rr;
    mx_rr_abp        = nanmax(abp_rr)/Fs;
end

ppg_beats = ann_ppg(n_ppg_beats);
ppg_rr = [];
ppg_bpm = [];
mx_rr_ppg = 0;
if ~isempty(ppg_beats)
    ppg_beats(end+1) = N_d;
    ppg_beats        = sort(unique(ppg_beats));
    ppg_rr           = diff(ppg_beats);
    ppg_rr(end+1)    = ppg_rr(end);
    ppg_bpm          = (60*Fs)./ppg_rr;
    mx_rr_ppg        = nanmax(ppg_rr)/Fs;
end

% SQI threshold
sqi_th = 0.95;

% Alarm threshold (seconds)
ASY_min_s_th = 4;
ASY_s_tol    = 0.5;

BRA_max_bpm_th = 40;
BRA_min_bts    = 4;

TAC_min_th  = 140;
TAC_min_bts = 14;

VTA_th = 100;

VFB_max_bpm_th = 150;
VFB_min_bpm_th = 300;
VFB_pwr_th     = 100;
VFB_min_bts    = 10;
tolerance  = 10; % tolerance = 5 bmp

sd_ecg = std(ecg2_sig_sect);
sd_th = 0.05;

sd_ecg5 = std(ecg5_sig_sect);

NaN_ecg2_sect = find(isnan(ecg2_signal(ecg2_beats)));
NaN_ecg5_sect = find(isnan(ecg5_signal(ecg5_beats)));

zero_ecg2_sect = find((ecg2_signal(ecg2_beats)) == 0);
zero_ecg5_sect = find((ecg5_signal(ecg5_beats)) == 0);

mx_len_sect = max ([length(ecg2_beats),length(ecg5_beats),length(abp_beats),length(ppg_beats)]);

% NaN_abp_sect = find(isnan(ann_abp(abp_beats)));
% NaN_ppg_sect = find(isnan(ann_ppg(ppg_beats)));

%%%%%%%%%%%% *** Signal Fusion *** %%%%%%%%%%%
% Start_signal_fusion = min (ecg2_beats(1),ecg5_beats(1),abp_beats(1),ppg_beats(1));
% End_signal_fusion = max (ecg2_beats(end),ecg5_beats(end),abp_beats(end),ppg_beats(end));
% 
% if (~isempty(ecg2_beats))
%     Signal_fusion_len(1) = length(ecg2_beats);
% elseif (~isempty(ecg5_beats))
%     Signal_fusion_len(2) = length(ecg5_beats);
% elseif (~isempty(abp_beats))
%     Signal_fusion_len(3) = length(abp_beats);
% elseif (~isempty(ppg_beats))
%     Signal_fusion_len(4) = length(ppg_beats);    
% end
% Signal_fusion_len = max(Signal_fusion_len);
% 
% for ii = Start_signal_fusion:Signal_fusion_len
    
switch alarm_type
    case 'Asystole'
        % No QRS for at least 4 seconds
        % if the signal quality is good enough and the maximum RR interval
        % is less than the Asystole threshold, set the alarm as 'F'
        
        %%%%%%%%%%%%%% Signals Fusion: 15/8/15 Nadi
        % Start with ECGII: First check the segments; if there were any NaN
        % beats for more than 2 beats, switch to ECGV, if there were also
        % more than 2 adjacent NaN beats, then switch to either ABP or PPG
        % available to use it for HR and make the decision. 
        % If there were NaN beats in more than half beats of a segment,
        % then switch to the other signal.
        % ????If there were no signals available for more than half of the
        % beats in the segment, then set the alarm as false!?????
   
        if  isempty(ecg2_rr_Outlier)
            if ~isempty(ecg2_beats) && sd_ecg > sd_th %&& length(zero_ecg2_sect)<=ceil(0.5*mx_len_sect) % && length(NaN_ecg2_sect)<=2 ||  isempty(NaN_ecg2_sect) %&& mx_rr_ecg2<6
                alarmResult=logical(mx_rr_ecg2>=ASY_min_s_th-ASY_s_tol);
            elseif ~isempty(ecg5_beats) && sd_ecg5 > sd_th %&& mx_rr_ecg5<6
                alarmResult=logical(mx_rr_ecg5>=ASY_min_s_th-ASY_s_tol);
            elseif ~isempty(abp_beats)
                alarmResult=logical(mx_rr_abp>=ASY_min_s_th-ASY_s_tol);
            elseif ~isempty(ppg_beats)
                alarmResult=logical(mx_rr_ppg>=ASY_min_s_th-ASY_s_tol);
            else
                alarmResult = 0;
            end
            
        else            %*** NaN beats in ECGII
            if ~isempty(ecg2_beats) && ~isempty(ecg2_rr_Outlier) %|| length(NaN_ecg2_sect)> 2
                % Measure the HR in non NaN beats of ECG
                % ecg2_beats_recons(NaN_ecg2_sect) = [];
                ecg2_beats_recons(1) = ecg2_beats(1);
                ii=2;
                % Switch from NaN beats of ECGs to non_NaN beats of next signal
                while ii<=length(ecg2_beats)-1
                    if ecg2_rr(ii)<=2*rr_mean
                        ecg2_rr_recons(ii) = ecg2_rr(ii);
                        mxrr_NaN(1) = nanmax([ecg2_rr_recons])/Fs;
                        ii=ii+1;
                        continue
                        %%%*** Look for those NaN beats in other signals
                    elseif ~isempty(ecg5_beats) && isempty(ecg5_rr_Outlier) && length(zero_ecg5_sect)<=ceil(0.5*mx_len_sect) %&& isempty(NaN_ecg5_beats)
                        ecg5_NotNaN = intersect(find(ecg5_beats>=ecg2_beats(ii)),find(ecg5_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ecg5_NotNaN)
                            ecg5_rr_recons = ecg5_rr(ecg5_NotNaN);
                            mxrr_NaN(2) = nanmax([ecg5_rr_recons])/Fs;
                            ii = ii+1;
                        else
                            ecg2_rr_recons(ii) = ecg2_rr(ii);
                            mxrr_NaN(1) = nanmax([ecg2_rr_recons])/Fs;
                            ii=ii+1;
                        end
                    elseif ~isempty(abp_beats) %&& length(NaN_abp_beats)>=2
                        abp_NotNaN = intersect(find(abp_beats>=ecg2_beats(ii)),find(abp_beats<=ecg2_beats(ii+1)));
                        if ~isempty(abp_NotNaN)
                            mxrr_NaN(3) = nanmax([abp_rr(abp_NotNaN)])/Fs;
                            ii = ii+1;
                        else
                            ecg2_rr_recons(ii) = ecg2_rr(ii);
                            mxrr_NaN(1) = nanmax([ecg2_rr_recons])/Fs;
                            ii=ii+1;
                        end
                    elseif ~isempty(ppg_beats) %&& length(NaN_abp_beats)>=2
                        ppg_NotNaN = intersect(find(ppg_beats>=ecg2_beats(ii)),find(ppg_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ppg_NotNaN)
                            mxrr_NaN(4) = nanmax([ppg_rr(ppg_NotNaN)])/Fs;
                            ii = ii+1;
                        else
                            ecg2_rr_recons(ii) = ecg2_rr(ii);
                            mxrr_NaN(1) = nanmax([ecg2_rr_recons])/Fs;
                            ii=ii+1;
                        end
                    else
                        % If all signals were zero or NaN; set the alarm to False
                        mxrr_NaN(5) = 1;
                        ii=ii+1;
                    end
                    % After calculating the HR from non-NaN beats of signals; measure the max HR
                    mx_rr = nanmax([mxrr_NaN(:)]);
                    alarmResult=logical(mx_rr>=ASY_min_s_th-ASY_s_tol);
                end
            end
        end
                
                    %             if ~isempty(ecg2_beats) && length(NaN_ecg2_sect)> 2
                        %                 % Measure the HR in non NaN beats of ECG
%                 ecg2_beats_recons(NaN_ecg2_sect) = [];
%                 mxrr_NaN(1) = max((diff(ecg2_beats_recons))/Fs);
%                 %*** Look for those NaN beats in other signals
%                 ecg5_NotNaN = intersect(find(ecg5_beats>=min(NaN_ecg2_sect)),find(ecg5_beats<=max(NaN_ecg2_sect)));
%                 ecg5_NotNaN_beats=ecg5_beats(ecg5_NotNaN);
%                 NaN_ecg5_beats = find(isnan(ecg2_signal(ecg5_NotNaN_beats)));
%                 
%                 abp_beats_NotNaN = intersect(find(abp_beats>=min(NaN_ecg2_sect)),find(abp_beats<=max(NaN_ecg2_sect)));
%                 NaN_abp_beats = abp_beats(abp_beats_NotNaN);
%                 
%                 ppg_beats_NotNaN = intersect(find(ppg_beats>=min(NaN_ecg2_sect)),find(ppg_beats<=max(NaN_ecg2_sect)));
%                 NaN_ppg_beats = ppg_beats(ppg_beats_NotNaN);             
%                 % Switch from NaN beats of ECGs to non_NaN beats of next signal
%                 if ~isempty(ecg5_beats) && isempty(NaN_ecg5_beats) && length(zero_ecg5_sect)<=ceil(0.5*mx_len_sect)
%                     mxrr_NaN(2) = max(diff(ecg5_beats(NaN_ecg5_beats))/Fs);
%                 elseif ~isempty(abp_beats) && length(NaN_abp_beats)>=2 
%                     mxrr_NaN(3) = max(diff(NaN_abp_beats))/Fs;
%                 elseif ~isempty(ppg_beats) && length(NaN_ppg_beats)>=2 
%                     mxrr_NaN(4) = max(diff(NaN_ppg_beats))/Fs;
%                 else     % If all signals were zero or NaN; set the alarm to False
%                     mxrr_NaN(5) = 1;
%                 end
%                 % After calculating the HR from non-NaN beats of signals; measure the max HR
%                  mx_rr = max(mxrr_NaN(:));
%                  alarmResult=logical(mx_rr>=ASY_min_s_th-ASY_s_tol);
% end


%                 alarmResult=logical(mx_rr_ecg2>=ASY_min_s_th-ASY_s_tol);
%             elseif ~isempty(ecg5_beats) && length(NaN_ecg5_sect)<=2
%                 alarmResult=logical(mx_rr_ecg5>=ASY_min_s_th-ASY_s_tol);
%             elseif ~isempty(abp_beats) && BEATQ>=sqi_th %&& length(NaN_abp_sect)<=2
%                 alarmResult=logical(mx_rr_abp>=ASY_min_s_th-ASY_s_tol);
%             elseif ~isempty(ppg_beats) && psqi>=sqi_th %&& length(NaN_ppg_sect)<=2
%                 alarmResult=logical(mx_rr_abp>=ASY_min_s_th-ASY_s_tol);
%             end
            
%         end
%         if (~isempty(ecg2_beats)) %&& mx_rr_ecg2<6) % && sd_ecg > sd_th)
%             alarmResult=logical(mx_rr_ecg2>=ASY_min_s_th-ASY_s_tol);
%         elseif (~isempty(ecg5_beats)) %&& mx_rr_ecg5<6) % && sd_ecg > sd_th)
%             alarmResult=logical(mx_rr_ecg5>=ASY_min_s_th-ASY_s_tol);
%         elseif ~isempty(abp_beats)
%             alarmResult=logical(mx_rr_abp>=ASY_min_s_th-ASY_s_tol);
%         elseif ~isempty(ppg_beats)
%             alarmResult=logical(mx_rr_ppg>=ASY_min_s_th-ASY_s_tol);
%         end
        
        %%%%Excerpt Starts here %%%%%
    case 'Bradycardia'
        % Heart rate lower than 40 bpm for 5 consecutive beats
        all_beats=[];
if  isempty(ecg2_rr_Outlier)
        if length(ecg2_beats)>=5 && (mx_rr_ecg2<6) && (mx_rr_ecg2>0)
            all_beats=ecg2_beats(1:end-1);
        elseif length(ecg5_beats)>=5 && (0<mx_rr_ecg5) && (mx_rr_ecg5<6)
            all_beats=ecg5_beats(1:end-1);
        elseif length(abp_beats)>=5 && abpsqi>=sqi_th
            all_beats=abp_beats(1:end-1);
        elseif length(ppg_beats)>=5 && ppgsqi>=sqi_th
            all_beats=ppg_beats(1:end-1);
        end

else            %*** NaN beats in ECGII
            if ~isempty(ecg2_beats) && ~isempty(ecg2_rr_Outlier) %|| length(NaN_ecg2_sect)> 2
                % Measure the HR in non NaN beats of ECG
                % ecg2_beats_recons(NaN_ecg2_sect) = [];
                all_beats(1) = ecg2_beats(1);
                ii=2;
                % Switch from NaN beats of ECGs to non_NaN beats of next signal
                while ii<=length(ecg2_beats)-1
                    if ecg2_rr(ii)<=2*rr_mean
                        all_beats(ii)= ecg2_beats(ii);
                        ii=ii+1;
                        continue
                        %%%*** Look for those NaN beats in other signals
                    elseif ~isempty(ecg5_beats) && isempty(ecg5_rr_Outlier) && length(zero_ecg5_sect)<=ceil(0.5*mx_len_sect) %&& isempty(NaN_ecg5_beats)
                        ecg5_NotNaN = intersect(find(ecg5_beats>=ecg2_beats(ii)),find(ecg5_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ecg5_NotNaN)
                            all_beats(ii)= ecg5_beats(ii);
                            ii = ii+1;
                        else
                            all_beats(ii) = ecg2_beats(ii);
                            ii=ii+1;
                        end
                    elseif ~isempty(abp_beats) %&& length(NaN_abp_beats)>=2
                        abp_NotNaN = intersect(find(abp_beats>=ecg2_beats(ii)),find(abp_beats<=ecg2_beats(ii+1)));
                        if ~isempty(abp_NotNaN)
                            all_beats(ii) = abp_beats(ii);
                            ii = ii+1;
                        else
                            all_beats(ii) = ecg2_beats(ii);
                            ii=ii+1;
                        end
                    elseif ~isempty(ppg_beats) %&& length(NaN_abp_beats)>=2
                        ppg_NotNaN = intersect(find(ppg_beats>=ecg2_beats(ii)),find(ppg_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ppg_NotNaN)
                            all_beats(ii) = ppg_beats(ii);
                            ii = ii+1;
                        else
                            all_beats(ii)= ecg2_beats(ii);
                            ii=ii+1;
                        end
                    else
                        % If all signals were zero or NaN; set the alarm to False
                        all_beats(ii)= nan;
                        ii=ii+1;
                    end
                end
            end
            all_beats(isnan(all_beats))=[];
            all_beats = sort(unique(all_beats));
end
        i=1;
        interval = 6.66*Fs;
        brady_found = 0;
        if ~isempty(all_beats)
            for i=1:length(all_beats)-5
                start = all_beats(i);
                beats = all_beats(all_beats>=start & all_beats<start+interval);
                if length(beats)<=5
                    brady_found=1;
                end
            end
        end
        alarmResult=logical(brady_found==1);
        %%%%Excerpt Ends here %%%%%
        
    case 'Tachycardia'
        % Heart rate higher than 140 bpm for 17 consecutive beats
        bpm_vec = [];
        if  isempty(ecg2_rr_Outlier)
            if  length(ecg2_beats)>=TAC_min_bts  &Y& (0<max_rr_ECG2) && (max_rr_ECG2<6)
                bpm_vec = ecg2_bpm;
            elseif  length(ecg5_beats)>=TAC_min_bts  && (0<max_rr_ECG5) && (max_rr_ECG5<6)
                bpm_vec = ecg5_bpm;
            elseif abpsqi>=sqi_th && length(abp_beats)>=TAC_min_bts
                bpm_vec = abp_bpm;
            elseif ppgsqi>=sqi_th && length(ppg_beats)>=TAC_min_bts
                bpm_vec = ppg_bpm;
            end
        else            %*** NaN beats in ECGII
            bpm_vec=[];
            if ~isempty(ecg2_beats) && ~isempty(ecg2_rr_Outlier) && length(ecg2_beats)>=TAC_min_bts
                % Measure the HR in non NaN beats of ECG
                % ecg2_beats_recons(NaN_ecg2_sect) = [];
                bpm_vec(1) = ecg2_bpm(1);
                ii=2;
                % Switch from NaN beats of ECGs to non_NaN beats of next signal
                while ii<=length(ecg2_bpm)-1
                    if ecg2_rr(ii)<=2*rr_mean
                            bpm_vec = [bpm_vec ecg2_bpm(ii)];
                            ii=ii+1;
                        continue
                        %%%*** Look for those NaN beats in other signals
                    elseif ~isempty(ecg5_beats) && isempty(ecg5_rr_Outlier) && length(zero_ecg5_sect)<=ceil(0.5*mx_len_sect) && length(ecg5_beats)>=TAC_min_bts%&& isempty(NaN_ecg5_beats)
                        ecg5_NotNaN = intersect(find(ecg5_beats>=ecg2_beats(ii)),find(ecg5_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ecg5_NotNaN)
                            
                            if size(ecg5_NotNaN,2)==1
                                ecg5_bpm = ecg5_bpm';
                            end
                            bpm_vec = [bpm_vec ecg5_bpm(ecg5_NotNaN)];
                            ii = ii+1;
                        else
                            bpm_vec = [bpm_vec  ecg2_bpm(ii)];
                            ii=ii+1;
                        end
                    elseif ~isempty(abp_beats) %&& length(NaN_abp_beats)>=2
                        abp_NotNaN = intersect(find(abp_beats>=ecg2_beats(ii)),find(abp_beats<=ecg2_beats(ii+1)));
                        if ~isempty(abp_NotNaN)
                            if size(abp_NotNaN,2)==1
                                abp_bpm = abp_bpm';
                            end
                            bpm_vec = [bpm_vec abp_bpm(abp_NotNaN)];
                            ii = ii+1;
                        else
                            bpm_vec = [bpm_vec ecg2_bpm(ii)];
                            ii=ii+1;
                        end
                    elseif ~isempty(ppg_beats) %&& length(NaN_abp_beats)>=2
                        ppg_NotNaN = intersect(find(ppg_beats>=ecg2_beats(ii)),find(ppg_beats<=ecg2_beats(ii+1)));
                        if ~isempty(ppg_NotNaN)
                            if size(ppg_NotNaN,2)==1
                                ppg_bpm = ppg_bpm';
                            end
                            bpm_vec = [bpm_vec ppg_bpm(ppg_NotNaN)];
                            ii = ii+1;
                        else
                            bpm_vec = [bpm_vec ecg2_bpm(ii)];
                            ii=ii+1;
                        end
                    else
                        % If all signals were zero or NaN; set the alarm to False
                        ii=ii+1;
                    end
                end
            end
            bpm_vec(isnan(bpm_vec))=[];
            bpm_vec = sort(unique(bpm_vec));
        end
        
        over_thr = bpm_vec(bpm_vec>=TAC_min_th-tolerance);
        if length(bpm_vec)>=30 || length(over_thr)>=TAC_min_bts
            alarmResult=1;
        else
            alarmResult=0;
        end
        
        % % Modified 11/08/15
        %         if max_rr_NaNECG2<6 && length(ecg2_NaNbeats)>=TAC_min_bts
        %             bpm_vec = ecg2_NaNbpm;
        %         elseif max_rr_NaNECG5<6 && length(ecg5_NaNbeats)>=TAC_min_bts
        %             bpm_vec = ecg5_NaNbpm;
        %         elseif abpsqi>=sqi_th && length(abp_beats)>=TAC_min_bts
        %             bpm_vec = abp_bpm;
        %         elseif ppgsqi>=sqi_th && length(ppg_beats)>=TAC_min_bts
        %             bpm_vec = ppg_bpm;
        %         end
        %         over_thr = bpm_vec(bpm_vec>=TAC_min_th-tolerance);
        %         if length(bpm_vec)>=30 || length(over_thr)>=TAC_min_bts
        %             alarmResult=1;
        %         else
        %             alarmResult=0;
        %         end
        
   %%%START%%%
    case 'Ventricular_Tachycardia'
        % 5 or more ventricular beats with heart rate higher than 100 bpm
%         if mx_rr_ecg2<4
%             alarmResult=vtach_detect(ecg2_sig_sect, ecg2_beats, N0_d,Fs);
%         else
%             alarmResult=vtach_detect(ecg5_sig_sect, ecg5_beats, N0_d,Fs);
%         end
        % Potential take this into a function
        i_sel_beat=[];
        i_sel_sigsec=[];
        if  isempty(ecg2_rr_Outlier)
            if ~isempty(ecg2_beat)
                i_sel_beat = ecg2_beat;
                i_sel_sigsec = ecg2_sig_sec;
            elseif ~isempty(ecg5_beat);
                i_sel_beat = ecg5_beat;
                i_sel_sigsec = ecg5_sig_sec;
            else
                i_sel_beat = [];
            end
        elseif  ~isempty(ecg2_beat) && ~isempty(ecg2_rr_Outlier) %&& length(ecg2_beats)>=TAC_min_bts
            
            i_sel_beat= [i_sel_beat ecg2_beat(1)];
            i_sel_sigsec = [i_sel_sigsec ecg2_sig_sec(1)];
            ii=2;
            % Switch from NaN beats of ECGs to non_NaN beats of next signal
            while ii<=length(ecg2_beat)-1
                if ecg2_rr(ii)<=2*rr_mean
                    i_sel_beat= [i_sel_beat ecg2_beat(ii)];
                    ii=ii+1;
                    continue
                    %%%*** Look for those NaN beats in other signals
                elseif ~isempty(ecg5_beat) && isempty(ecg5_rr_Outlier) && length(zero_ecg5_sec)<=ceil(0.5*mx_len_sec) %&& isempty(NaN_ecg5_beats)
                    ecg5_NotNaN = intersect(find(ecg5_beat>=ecg2_beats(ii)),find(ecg5_beat<=ecg2_beat(ii+1)));
                    if ~isempty(ecg5_NotNaN)
                        i_sel_beat= [i_sel_beat ecg5_beat(ii)];
                        ii = ii+1;
                    else
                        all_beats(ii) = ecg2_beats(ii);
                        ii=ii+1;
                        %                     elseif ~isempty(ecg5_beat);
                        %                         i_sel_beat = ecg5_beat;
                        %                         i_sel_sigsec = ecg5_sig_sec;
                    end
                else
                    i_sel_beat = [];
                end
            end
        end
        
        
%              all_beats(1) = ecg2_beats(1);
%                 ii=2;
%                 % Switch from NaN beats of ECGs to non_NaN beats of next signal
%                 while ii<=length(ecg2_beats)-1
%                     if ecg2_rr(ii)<=2*rr_mean
%                         all_beats(ii)= ecg2_beats(ii);
%                         ii=ii+1;
%                         continue
%                         %%%*** Look for those NaN beats in other signals
%                     elseif ~isempty(ecg5_beats) && isempty(ecg5_rr_Outlier) && length(zero_ecg5_sect)<=ceil(0.5*mx_len_sect) %&& isempty(NaN_ecg5_beats)
%                         ecg5_NotNaN = intersect(find(ecg5_beats>=ecg2_beats(ii)),find(ecg5_beats<=ecg2_beats(ii+1)));
%                         if ~isempty(ecg5_NotNaN)
%                             all_beats(ii)= ecg5_beats(ii);
%                             ii = ii+1;
%                         else
%                             all_beats(ii) = ecg2_beats(ii);
%                             ii=ii+1;
%                         end
%                     elseif ~isempty(abp_beats) %&& length(NaN_abp_beats)>=2
%                         abp_NotNaN = intersect(find(abp_beats>=ecg2_beats(ii)),find(abp_beats<=ecg2_beats(ii+1)));
%                         if ~isempty(abp_NotNaN)
%                             all_beats(ii) = abp_beats(ii);
%                             ii = ii+1;
%                         else
%                             all_beats(ii) = ecg2_beats(ii);
%                             ii=ii+1;
%                         end
%                     elseif ~isempty(ppg_beats) %&& length(NaN_abp_beats)>=2
%                         ppg_NotNaN = intersect(find(ppg_beats>=ecg2_beats(ii)),find(ppg_beats<=ecg2_beats(ii+1)));
%                         if ~isempty(ppg_NotNaN)
%                             all_beats(ii) = ppg_beats(ii);
%                             ii = ii+1;
%                         else
%                             all_beats(ii)= ecg2_beats(ii);
%                             ii=ii+1;
%                         end
%                     else
%                         % If all signals were zero or NaN; set the alarm to False
%                         all_beats(ii)= nan;
%                         ii=ii+1;
%                     end
%                 end
%             end
%             all_beats(isnan(all_beats))=[];
%             all_beats = sort(unique(all_beats));
% end
            if ~isempty(i_sel_beat)
            sd                = std(i_sel_sigsec);
            temp_beats        = i_sel_beat-N0_d+1;
            temp_dif          = diff(temp_beats);
            compare           = temp_dif(1);
            re_search         = [];
            for i=2:length(temp_dif)
                this = temp_dif(i);
                if this >= 1.5*compare
                    re_search(end+1) = i;
                end
                compare = this;
            end
            
            to_add = [];
            for i=re_search
                i_start       = temp_beats(i);
                i_end         = temp_beats(i+1);
                rs_snip       = i_sel_sigsec(i_start:i_end);
                rs_sd         = std(rs_snip);
                rs_mean       = mean(rs_snip);
                t             = maxdet(rs_snip, rs_sd);
                t             = t(rs_snip(t)>=rs_mean+sd);
                t             = t+i_start+N0_d-1;
                to_add        = [to_add, t];
            end
            
            temp       = [i_sel_beat(1:end-1)+1, to_add];
            temp       = sort(unique(temp));
            i_sel_beat = temp-N0_d;
            
            % Set template based on first waveform
            mx_a     = i_sel_beat(1);
            mx_b     = i_sel_beat(2);
            snip     = i_sel_sigsec(mx_a:mx_b-1);
            mnsn     = mean(snip);
            % Remove Mean
            template = snip-mnsn;
            
            filtvec = [];
            for i=2:length(i_sel_beat)-1
                snip = i_sel_sigsec(i_sel_beat(i):i_sel_beat(i+1)-1);
                mnsn = mean(snip);
                snip = snip-mnsn;
                insig = resample(snip,length(template),length(snip));
                cc    = std(abs(template-insig));
                if cc >=0.6*sd
                    filtvec(end+1) = cc;
                end
            end
            if length(filtvec)>=4
                alarmResult = 1;
            else
                alarmResult = 0;
            end
        else
            alarmResult = 0;
        end
%%%END%%%

        
    case 'Ventricular_Flutter_Fib'
        % Fibrillatory, flutter, or oscillatory waveform for at least 4 seconds
        %%% START OF EXCERPT    %%% Added 16/8/15 Jacqui
        if 0<max_rr_ECG2 && max_rr_ECG2<6 && length(n_ECG2_beats)>=VFB_min_bts
            bpm_vec = ecg2_bpm(1:end-1); % exclude the appended final beat
        elseif 0<max_rr_ECG5 && max_rr_ECG5<6 && length(n_ECG5_beats)>=VFB_min_bts
            bpm_vec = ecg5_bpm(1:end-1); % exclude the appended final beat
        elseif length(n_abp_beats)>=VFB_min_bts
            bpm_vec = abp_bpm(1:end-1); % exclude the appended final beat
        elseif length(n_ppg_beats)>=VFB_min_bts
            bpm_vec = ppg_bpm(1:end-1); % exclude the appended final beat
        else
            bpm_vec = nan;
        end
     
        i_ecg_signal=[];
        if ~isempty(ecg2_sig_sect) && max(ecg2_sig_sect)<inf
            i_ecg_signal=ecg2_sig_sect;
        elseif ~isempty(ecg5_sig_sect) && max(ecg5_sig_sect)<inf
            i_ecg_signal=ecg5_sig_sect;
        end
            
        % Trying to oscillatory waveform in 10-20 Hz Freq range
        if ~isempty(i_ecg_signal)
            [spec,sf] = specgram(i_ecg_signal,Fs);
            power     = powercalc(spec,10,20,sf);

            alarmResult=1;
            if power<VFB_pwr_th
                alarmResult=0;
            end
        else
            alarmResult=logical(max(bpm_vec)>VFB_min_bpm_th-tolerance);
        end

%%% END OF EXCERPT %%%


%         if length(n_ECG2_beats)>=VFB_min_bts % && max_rr_ECG2<6
%             bpm_vec = ecg2_bpm(1:end-1); % exclude the appended final beat
%         elseif length(n_ECG5_beats)>=VFB_min_bts % && max_rr_ECG2<6
%             bpm_vec = ecg5_bpm(1:end-1); % exclude the appended final beat
%         elseif length(n_abp_beats)>=VFB_min_bts
%             bpm_vec = abp_bpm(1:end-1); % exclude the appended final beat
%         elseif length(n_ppg_beats)>=VFB_min_bts
%             bpm_vec = ppg_bpm(1:end-1); % exclude the appended final beat
%             
%             %Added 14/8/15 Nadi for record 431 without any decent signal!
%         else
%             bpm_vec = 0;
%             
%         end
%         
%         % Trying to oscillatory waveform in 10-20 Hz Freq range
%         [spec,sf] = specgram(ecg2_sig_sect,Fs);
%         power     = powercalc(spec,10,20,sf);
%         
%         if max(bpm_vec)<=VFB_max_bpm_th
%             alarmResult=0;
%         elseif max(bpm_vec)>=VFB_min_bpm_th-tolerance
%             alarmResult=1;
%         elseif power<VFB_pwr_th
%             alarmResult=0;
%             %Added 14/8/15 Nadi for record 431 without any decent signal!
%         else
%             alarmResult=0;
%         end
        
    otherwise
        error(['Unknown alarm type: ' alarm_type])
end

% Write result to answers.txt
fid = fopen(answers, 'a');
if (fid == -1)
    error('Could not open answer file');
end

% Get base name of record (without directories)
i = strfind(recordName, filesep);
if (~isempty(i))
    basename = recordName(i(end)+1 : end);
else
    basename = recordName;
end

fprintf(fid, '%s,%d\n', basename, alarmResult);
fclose(fid);

end

function vtach_result=vtach_detect(signal_sect, mxs, start_idx, Fs)
mxs        = mxs-start_idx+1;
% make sure peaks are located at local maxima
pos        = chk_peaks(signal_sect, mxs, 5, 0.2*Fs);
no_p_count = 0;
% Define buffer of 100+/-20ms from each R peak
qr_width   = round(0.12*0.5*Fs);
rs_width   = qr_width;
for i=1:length(pos)-1
    mx_a   = pos(i);
    mx_b   = pos(i+1);
    snip   = signal_sect(mx_a:mx_b);
    indent = mx_a;
    rr_int = mx_b-mx_a;
    bts    = Fs*60/rr_int;
    
    if rr_int>qr_width*3
        snip   = snip(rs_width:end-qr_width);
        indent = mx_a+rs_width;
    end
    sd_snip = std(snip);
    
    bnd_a   = (rr_int/Fs)*30+mx_a;
    bnd_b   = (rr_int/Fs)*135+mx_a;
    
    mx_bnd  = min([signal_sect(mx_a), signal_sect(mx_b)]);
    pk      = maxdet(snip,0.05*sd_snip);
    pk      = pk(pk>1);
    pk      = pk(snip(pk)<mx_bnd);
    peaks   = pk+indent;
    bnd_pks = peaks(bnd_a<peaks & peaks<bnd_b);
    
    found_nopwave = 0;
    if isempty(peaks)
        found_nopwave = 1;
    elseif length(peaks)==1
        found_nopwave = 0;
    elseif bts<50 % todo: should exclude this, implement back-search when R peak not found for X seconds
        found_nopwave = 5;
    elseif length(peaks)>1 && ~isempty(bnd_pks)
        found_nopwave = 0;
    end
    no_p_count = no_p_count+found_nopwave;
end
vtach_result=logical(no_p_count>=4); % once function included to main script, define variable = 4
end

function rpeaks=chk_peaks(win_signal, m, window, winsearch)
rpeaks = [];
maxvec = m(m>winsearch & m<length(win_signal)-winsearch);
for mx=maxvec
    mx   = int64(mx);
    pre  = mean(win_signal(mx-window:mx));
    post = mean(win_signal(mx:mx+window));
    if pre<win_signal(mx) && post<win_signal(mx)
        rpeaks(end+1)=mx;
    else
        r_found=false;
        delta=0.25;
        if pre<post
            start=mx;
            first=true;
        else
            start=mx-winsearch+1;
            first=false;
        end
        search=win_signal(start:start+winsearch);
        while ~r_found && delta>0
            rvec=maxdet(search, delta);
            r_found=logical(length(rvec));
            if r_found
                if first
                    rpeaks(end+1)=start+rvec(1);
                else
                    rpeaks(end+1)=start+rvec(end);
                end
            else
                delta=delta-0.05;
            end
        end
    end
end
if ~isempty(rpeaks)
    rpeaks = sort(unique(rpeaks));
    dif    = [window+1,diff(rpeaks)];
    rpeaks = rpeaks(dif>2);
end
end

function [spec,sf]=specgram(sig, Fs)
sig     = resample(sig,5*Fs, length(sig));
sum_sig = sum(sig.^2);
scl_sig = sig/sqrt(sum_sig);
spec    = fft(scl_sig);
spec    = abs(spec(1:int64(length(spec)/2)));
sf      = int64(length(spec)/(Fs/2));
end

function power=powercalc(specg, bound_a, bound_b, sf)
power = sum(specg(bound_a*sf:bound_b*sf).^2);
end
%%%%%%%%%%%% Helper Functions %%%%%%%%%%%%%%%%%%%%%
function ind=get_index(description,pattern)
M=length(description);
tmp_ind=strfind(description,pattern);
ind=[];
for m=1:M
    if(~isempty(tmp_ind{m}))
        ind(end+1)=m;
    end
end
end
%***************  Delay of signals and mapping to ECG QRSs  **************
% Measure the delay between qrs detections of signals & map the signal to QRS detections of ECG
function [foundMatch, rrDiff, peaks_signal] = qrs_map (peaks_signal, qrs_ecg, Fs)
if size(peaks_signal,2)~=size(qrs_ecg,2)
    qrs_ecg = qrs_ecg';
end
%         N_START = max(qrs_signal1(1),qrs_signal2(1));
%         N_Finish = min(qrs_signal1(end),qrs_signal2(end));
N_WINDOW = 60*Fs; % 60 seconds segments
N_START = 0; % seconds
% move initialisation fo foudnMatch in case ann_qrs is empty
foundMatch=false;
if isempty( qrs_ecg)
    peaks_signal = peaks_signal-0.2;
    idxRem = peaks_signal<=0;
    peaks_signal(idxRem) = [];
    if nargin>2
        sq(idxRem,:)=[];
    end
    rrDiff = 0.2;
    return;
end
foundMatch=false;
while N_START < N_Finish-N_WINDOW && foundMatch == false
    N_END = N_START + N_WINDOW;
    idxKeep =find( qrs_ecg>=(N_START) & qrs_ecg<=(N_END));
    ann_qrs_map =qrs_ecg(idxKeep);
    
    idxKeep = find(peaks_signal>=(N_START) & peaks_signal<=(N_END));
    ann_abp_map = peaks_signal(idxKeep);
    
    %             if numel(ann_abp_map)<10 || numel(ann_qrs_map)<10
    %                 N_START = N_START + N_WINDOW;
    %                 continue;
    %             end
    % no QRS annotations within ABP annotations
    %             if all(ann_abp_map < ann_qrs_map(1) | ann_abp_map > ann_qrs_map(end))
    %                 N_START = N_START + N_WINDOW;
    %                 continue;
    %             end
    %
    % make sure ann_gqrs circumscribes wabp
    %             while ann_abp_map(1)<ann_qrs_map(1)
    %                 ann_abp_map(1) = [];
    %             end
    %
    %             while ann_abp_map(end)>ann_qrs_map(end)
    %                 ann_abp_map(end) = [];
    %             end
    
    [idxGood,idxMatch] = histc(ann_abp_map,ann_qrs_map);
    idxMatch = idxMatch+1;
    idxGood(idxGood>1) = 0;
    
    if mean(idxGood)>0.9 % at least 10 good matches
        foundMatch = true;
        % this works, a lot of WABP match QRS
    else
        N_START = N_START + N_WINDOW;
    end
end
%map ABP or PPG to QRS
if foundMatch==false
    rrDiff = floor(0.2*Fs); % 200ms as a rough guess
else
    ann_qrs_map = ann_qrs_map(idxMatch); % maps QRS to matching WABPs to the right of it
    [ann_qrs_map,idxUniq] = unique(ann_qrs_map);
    ann_abp_map = ann_abp_map(idxUniq); % remove double WABP detections if present
    rrDiff = floor(median(ann_abp_map-ann_qrs_map));
end

peaks_signal = peaks_signal - rrDiff;
peaks_signal = peaks_signal(:);
idxRem = peaks_signal<=0;
peaks_signal(idxRem) = [];

end
%%%%%%%%%%%%%%%%%%%%%%%%%% ECG Signal Quality Index %%%%%%%%%%%%%%%%%%%%%%%
function [ sqi, tsqi ] = ecgsqi( ann1, ann2, REG_WIN, SIZE_WIND, LG_MED, THR )
% THR = 0.15; % window for matching two peaks
% LG_MED = 3; % take the median SQI across X seconds
% SIZE_WIND = 10;
% REG_WIN = 1; % one window per second
% HALF_WIND = opt_default.SIZE_WIND/2;
% LG_REC = ceil(max(max(ann1),max(ann2))); % length of the record in sample
% if isnan(LG_REC)  % length of the record in samples
%     % if it not provided, we replace it with our best guess using ann
%     %     LG_REC = length(ann1);
%     LG_REC = length(ann1);
%     LG_REC = ceil(max(max(ann1),max(ann2)));
%     N_WIN = ceil(LG_REC/REG_WIN); % number of windows in the signal
% end
LG_REC = length(ann1);
N_WIN = ceil(LG_REC/REG_WIN); % number of windows in the signal
% Use histc to bin each ABP/PPG peaks into bins centered on ECG qrs
xi = [ann1' - THR;
    ann1' + THR];
xi = xi(:);

% we now replace monotonically non-decreasing entries with their avg
% normally, we have two ECG beats next to each other, and windows do
% not overlap, i.e.:   |  x  |     l  x  l

% where 'x' is a beat with || and ll define the +- window

% sometimes the beats are too close however, and the windows overlap
% the following code changes windows like this:  |  l |  l
% to windows like this:                          |   |l  l

idxFix = [false;diff(xi) < 0];
xi_fixed = [xi(idxFix),xi([idxFix(2:end);false])];
xi_fixed = mean(xi_fixed,2);

xi(idxFix) = xi_fixed;
xi([idxFix(2:end);false]) = xi_fixed;

%=== now place gqrs into bins centered on jqrs
% First check if there are any non-decreasing edges and remove them
xi=sort(xi);
NonDec_edges=find(diff(xi) <= 0);
xi(NonDec_edges)=[];
N_J = histc(ann1,xi);

%=== repeat for the other way round
xi = [ann2' - THR;
    ann2' + THR];
xi = xi(:);
idxFix = [false;diff(xi) < 0];
xi_fixed = [xi(idxFix),xi([idxFix(2:end);false])];
xi_fixed = mean(xi_fixed,2);

xi(idxFix) = xi_fixed;
xi([idxFix(2:end);false]) = xi_fixed;
% xi=sort(xi);
% NonDec_edges=find(diff(xi) <= 0);
N_G = histc(ann2,xi);


% for each peak in ECG QRS, N_G indicates whether it has a matching peak
% in ABP/PPG (and vice versa for N_J)
N_J = N_J(1:2:end);
N_G = N_G(1:2:end);
N_J = N_J(:);
N_G = N_G(:);

% now bin APB or PPG peaks/ ECG peaks into the SQI windows

% create windows for the SQI
LG_REC = length(ann1);
% LG_REC = ceil(max(max(ann1),max(ann2)));
N_WIN = ceil(LG_REC/REG_WIN);
xi1 = (0:REG_WIN:LG_REC)';
xi2 = xi1+SIZE_WIND;
F1_1 = zeros(N_WIN,1);
F1_2 = zeros(N_WIN,1);

for w=1:numel(xi1)
    idx1 = ann1>xi1(w) & ann1<xi2(w);
    idx2 = ann2>xi1(w) & ann2<xi2(w);
    F1_1(w) = mean( N_J(idx1)==1 );
    F1_2(w) = mean( N_G(idx2)==1 );
end

% We take the minimum of these two to ensure we have a good match
F1 = min(F1_1,F1_2);
% NaN values indicate no ECG data for either lead - definitely bad quality
F1(isnan(F1)) = 0;

% Remove the non-relevant segments
idxRem = xi1 >= LG_REC;
F1(idxRem) = [];
xi1(idxRem) = [];

%Now smooth the SQI (F1)
if size(F1,1) < (LG_MED*2+1)
    F1smooth = F1;
else
    F1smooth = nan(size(F1,1),2*LG_MED+1);
    for k=1:LG_MED
        % create a lagged version of F1
        F1smooth(:,k) = vertcat(ones(k,1),F1(1:end-k));
        
        % create a led version of F1
        F1smooth(:,k+LG_MED) = vertcat(F1(k+1:end),ones(k,1));
    end
    F1smooth(:,end) = F1;
    % take the min in a window of 3 samples around the current F1 score
    F1smooth = min(F1smooth,[],2);
end
sqi = F1smooth;
tsqi = xi1;
end

% *********************** NaN Modification *******************************
function [RpeaksMod_ecg] = ecg_NaNModify(ecg, Fs)
% function [iNaN, RpeaksMod_ecg, RpeaksNaN, ECG_NaN, ecg_restored] = ecg_NaNModify(ecg, Fs)
% Check for NaN values in the ECGs and replace them with zero(previous part of
% ECG (mirroring)), then apply Hilbert QRS detection (as the Hilbert
% algorithm stops when gets to NaN) 23/7/15 Nadi

iNaN = find(isnan(ecg));
if isempty(iNaN)
    RpeaksMod_ecg = QRSdet_Hilbert(ecg',Fs);
else
    ecg(iNaN)= 0;
    RpeaksMod_ecg = QRSdet_Hilbert(ecg',Fs);
end

ECG2_NaN = ecg;
% Removing the replaced zeroes from ECG signal and R peaks after QRS detection
ecg(iNaN)= NaN;
% In some records e.g. 123; 4 detected Rpeaks are over ECG length!: So any Rpeaks over ECG length removed
Rpeaks_OverECG = find (RpeaksMod_ecg > length(ecg));
RpeaksMod_ecg(Rpeaks_OverECG) = [];
RpeaksNaN = RpeaksMod_ecg;
% Remove the Rpeaks falling in the NaN ECG segments
kk = find (isnan(ecg(RpeaksMod_ecg)));
RpeakssMod_ecg(kk) = [];
end
