clear all;close all;clc
data_dir=[pwd filesep];

%Add the function on this directory to the MATLAB path
%This is not permanent. This change is only valid for this session of
%MATLAB (will reset once MATLAB is restarted).
addpath(pwd)

if(exist('OCTAVE_VERSION'))
    more off %this seems necessary in order to get back the screen in Octave, but we have not tested this script on Octave yet.
end

%Check for previous files before starting test
data=dir(['vtach.csv']);
if(~isempty(data))
    while(1)
        display(['Found previous answer sheet file in:' pwd])
        cont=input('Delete it (Y/N/Q)?','s');
        if(strcmp(cont,'Y') || strcmp(cont,'N') || strcmp(cont,'Q'))
            if(strcmp(cont,'Q'))
                display('Exiting script!!')
                return;
            end
            break;
        end
    end
    if(strcmp(cont,'Y'))
        display('Removing previous answer sheet.')
        delete(data.name);
    end
end

fprintf('Generating validation set, please wait...\n')
fid=fopen([data_dir 'ALARMS'],'r');
if(fid ~= -1)
    RECLIST=textscan(fid,'%s %s %d','Delimiter',',');
    fclose(fid);
else
    error('Could not open vtach.csv for scoring. Exiting...')
end

matrix=zeros(16*125,341);
peaks_mat=zeros(50,341);
peaks_mat2=zeros(50,341);
peaks_mat3=zeros(50,341);
meta_mat=zeros(1,341);
RECORDS=RECLIST{1};
ALARMS=RECLIST{2};
GOLD=RECLIST{3};

%Specific to particular ALARM
index=find(ismember(ALARMS,'Ventricular_Tachycardia'));
RECORDS=RECORDS(index);
ALARMS=ALARMS(index);
GOLD=GOLD(index);

N=length(RECORDS);
results=zeros(N,1);

mark_5m = 125*60*5;

total_time=0;
for i=1:N
    recno = index(i);
    fprintf('**Scoring complete for: %d\n', i)

    % Name of file containing answers
    data = 'vtach.csv';

    %Get all ECG, blood pressure and photoplethysmogram signals
    recordName=RECORDS{i};
    gold=GOLD(i);
    alarm_type=ALARMS{i};
    
    [~,signal,Fs,siginfo]=rdmat(recordName);
    alarmResult=1;

    % % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % % %%%%%%%%%%% Nadi 18/3/2015
    % % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    description=squeeze(struct2cell(siginfo));
    description=description(4,:);

    % Resample signal to 125Hz
    Fs=Fs(1);
    if Fs~=125
        signal=resample(signal,125,Fs);
        Fs=125;
    end

    % Adding ECG signals
    ecg_signal  = signal(:,1);
    ecg_signal  = medfilt1(ecg_signal);    
    ecg_snip    = ecg_signal(mark_5m-1999:mark_5m);
    
    matrix(:,i) = ecg_snip;
    Rpeaks      = QRSdet_Hilbert(ecg_signal',Fs);

    meta_mat(1,i)=gold;
    
    %Run WABP on the record, which by default will analyze the first ABP, ART, or BP signal
    N=[];
    N0=[];
    abp_ind=get_index(description,'ABP');
    ann_abp=[];
    features=[];
    BEATQ=[];
    R=[];
    if(~isempty(abp_ind))
       ann_abp=wabp(signal(:,abp_ind),0,1);
       % Analyze the signal quality index of ABP using jSQI
       if length(ann_abp)>=3 % at least 3 abp beats detected
            [features] = abpfeature(signal(:,abp_ind),ann_abp);
            [BEATQ R] = jSQI(features, ann_abp, signal(:,abp_ind));
       end
    end

    %Run WABP on the record of 'PLETH' to analyze photoplethysmogram signal
    ppg_ind=get_index(description,'PLETH');
    ann_ppg=[];
    if (~isempty(ppg_ind))
        y=quantile(signal(:,ppg_ind),[0.05,0.5,0.95]);
        ann_ppg=wabp(signal(:,ppg_ind),0,(y(3)-y(1))/120);
        % Analyze the signal quality index of PPG 
        if ~isempty(ann_ppg)
            [psqi]=ppgSQI(signal(:,ppg_ind),ann_ppg);
        end
    end

    %Make decisions

    % set valid data segment for decision making, 16s before the alarm
    N_d=Fs*5*60; % alarm position
    N0_d=N_d-Fs*16+1; % 16s before the alarm

    % select the beats in the segment
    n_abp_beats=intersect(find(ann_abp>=N0_d),find(ann_abp<=N_d));
    n_ppg_beats=intersect(find(ann_ppg>=N0_d),find(ann_ppg<=N_d));
    %Added ECG II
    n_ECG2_beats=intersect(find(Rpeaks>=N0_d),find(Rpeaks<=N_d));
    %Added ECG V
    % n_ECG5_beats=intersect(find(Rpeaks5>=N0_d),find(Rpeaks5<=N_d));

    hr_max_abp=NaN;
    max_rr_abp=NaN;
    max_rr_abp=NaN;
    hr_max_ppg=NaN;
    max_rr_ppg=NaN;
    hr_max_ECG2=NaN;
    max_rr_ECG2=NaN;
    no_p_wave_cnt = 0;

    % calculate the heart rate
    if length(n_abp_beats)>=2
        hr_max_abp=60*Fs/min(diff(ann_abp(n_abp_beats)));
        max_rr_abp=max(diff(ann_abp(n_abp_beats)))/Fs;
    end
    if length(n_ppg_beats)>=2
        hr_max_ppg=60*Fs/min(diff(ann_ppg(n_ppg_beats)));
        max_rr_ppg=max(diff(ann_ppg(n_ppg_beats)))/Fs;
    end
    %Added ECG II
    if length(n_ECG2_beats)>=2
        hr_max_ECG2=60*Fs/min(diff(Rpeaks(n_ECG2_beats)));
        max_rr_ECG2=max(diff(Rpeaks(n_ECG2_beats)))/Fs;
    end

    peaks_mat(1:length(Rpeaks(n_ECG2_beats)),i) = Rpeaks(n_ECG2_beats);
    peaks_mat2(1:length(ann_abp(n_abp_beats)),i) = ann_abp(n_abp_beats);
    peaks_mat3(1:length(ann_ppg(n_ppg_beats)),i) = ann_ppg(n_ppg_beats);
    
    %rough P-wave detection for VTach
    l_ECG2_window=Fs*0.2; %Search in window of 200 ms
    pwave_th = 0.1;
    
end

csvwrite('ecg_data_vt.csv',matrix);
csvwrite('ecg_peaks_vt.csv',peaks_mat);
% csvwrite('abp_peaks_vtach.csv',peaks_mat2);
% csvwrite('ppg_peaks_vtach.csv',peaks_mat3);
csvwrite('ecg_meta_vt.csv',meta_mat);

fprintf(['**Scoring complete.\n'])